unit EZMapProjectionError;

interface

uses
  SysUtils, Classes;

type
  TErrorListing = array [0..999,0..2] of Smallint;
  TErrorText    = array [0..38] of string;
  TErrorRoutine = array [0..147] of string;
  TErrorProject = array [0..64] of string;
  TErrorCode    = record
    Routine     : string;
    Projection  : string;
    Error       : string;
  end;

const
  ErrorText : TErrorText =
	('Course between antipodal points is undefined.',
	'Algorithm did not converge.',
	'Only N-S courses are meaningful, starting at a pole.',
	'No Flight when windspeed exceeds flightspeed.',
	'No flight at a speed of zero.',
	'Both coordinates are the same.',
	'Ellipsoid is not valid.',
	'Zone number is not of proper type.',
	'EllipsoidID does not exist. EllipsoidID is set to zero.',
	'DatumID does not exist. DatumID is set to zero.',
	'GridID does not exist. GridID is set to zero.',
	'EllipsoidID does not exist. EllipsoidID is set to zero.','',
        'DatumID does not exist. DatumID is set to zero.','',
	'GridID does not exist. GridID is set to zero.','','','',
	'Coordinates out of DutchGrid range.',
	'Wrong latitude input. Position outside projection range.',
	'Wrong longitude input. Position outside projection range.',
	'Position outside projection range.',
	'Point is not projectable due to convergence error.',
	'Point is not projectable. Map constants might not be apropriate.',
	'Too many iterations. Could not project point.',
	'Equal Latitiudes for St. Parallels on opposite sides of equator.',
	'Point is not projectable.',
	'Radius is to small.',
	'Inappropriate projection constants or the radius is to small.',
	'Reverse computation is at this moment not possible.',
	'No sufficient reference points because some points coincide.',
	'No sufficient reference points.',
	'Height is smaller than 0.','','','',
	'Library has not been unlocked.',
	'Library has not been loaded or routine could not be found.');

  ErrorRoutine : TErrorRoutine =
	('Translate2','CalcDistance','CalcEndpoint','CalcFlightCoars','GetZone','GeoToUTM','GeoToDatumUTM',
	'GeoToNationalGrid','UTMtoGeo','DatumUTMtoGeo','NationalGridToGeo','NLToRD','RDToNL','UTMRef','OrthFor',
	'OrthInv','WinkelIFor','WinkelIInv','BonneFor','BonneInv','UTMFor','UTMInv','WinkelIIFor','WinkelIIInv',
	'SinusoidalFor','SinusoidalInv','EquirectFor','EquirectInv','CalculateRef','BStereoFor','BStereoInv',
	'EquirectRef','GvnspFor','GvnspInv','HammerFor','HammerInv','AzimuthalEAFor','AzimuthalEAInv','LaAzimuthalEAFor',
	'LaAzimuthalEAInv','GrintenFor','GrintenInv','PolyconicFor','PolyconicInv','RobinsonFor','RobinsonInv',
	'LaCConicFor','LaCConicInv','GnomonicFor','GnomonicInv','TMercatorFor','TMercatorInv','CassiniSFor',
	'CassiniSInv','MercatorFor','MercatorInv','MollweideFor','MollweideInv','WagnerIVFor','WagnerIVInv',
	'WagnerVIIFor','WagnerVIIInv','IMollweideFor','IMollweideInv','ConicalEDFor','ConicalEDInv','AlbertsCEAFor',
	'AlbertsCEAInv','StereographicFor','StereographicInv','EckertTFor','EckertTInv','GetMapHeading','LamCylindricalEAFor',
	'LamCylindricalEAInv','BerCylindricalEAFor','BerCylindricalEAInv','TristanEdwardsFor','TristanEdwardsInv',
	'PetersFor','PetersInv','GallOrthFor','GallOrthInv','BalthasartFor','BalthasartInv','CollignonFor',
	'CollignonInv','EckertIFor','EckertIInv','EckertIIIFor','EckertIIIInv','PutninsFor','PutninsInv','KavraiskyVIIFor',
	'KavraiskyVIIInv','WagnerVIFor','WagnerVIInv','EckertVFor','EckertVInv','CrasterFor','CrasterInv',
	'EckertIVFor','EckertIVInv','FaheyFor','FaheyInv','FoucautSFor','FoucautSInv','GallFor','GallInv',
	'EckertVIFor','EckertVIInv','McBrydeFor','McBrydeInv','LoximuthalFor','LoximuthalInv','MillerFor',
	'MillerInv','NellFor','NellInv','FoucautFor','FoucautInv','KavraiskyVFor','KavraiskyVInv','McBrydeSinFor',
	'McBrydeSinInv','QuarticAuFor','QuarticAuInv','WagnerIIFor','WagnerIIInv','WagnerIIIFor','WagnerIIIInv',
	'UrmaevFor','UrmaevInv','WagnerIFor','WagnerIInv','UrmaevVFor','UrmaevVInv','McBrydeSin2For','McBrydeSin2Inv',
	'McBrydePFor','McBrydePInv','McBrydeQFor','McBrydeQInv','CalcRef','FromTo','CalculateFor','CalculateInv',
	'GetDeclination');

  ErrorProject : TErrorProject =
	('UTM Grid','UTM Lokal Grid','National Grid','Alberts Conical Equal-Area','Azimuthal Equal Area','Balthasart',
	'Behrmann cylindrical equal-area','Bonne','Brauns Stereographic Cylindrical','Cassini-Soldner','Collignon',
	'Craster (Putnins P4)','Eckert I','Eckert II','Eckert III','Eckert IV','Eckert V','Eckert VI','Equidistant Conical',
	'Equirectangular (Plate Carree)','Fahey','Foucaut','Foucaut Sinusoidal','Gall orthographic','Gall Stereographic',
	'General Vertical Near-side Perspective','Gnomonic','Hammer','Interupted Mollweide','Kavraisky V','Kavraisky VII',
	'Lambert Azimuthal Equal Area','Lambert Conformal Conic','Lambert cylindrical equal-area projection','Loximuthal',
	'McBryde-Thomas Flat-Polar Parabolic','McBryde-Thomas Flat-Polar Quartic','McBryde-Thomas Flat-Polar Sine (No. 1)',
	'McBryde-Thomas Flat-Polar Sine (No. 2)','McBryde-Thomas Flat-Polar Sinusoidal','Mercator','Miller Cylindrical',
	'Mollweide','Nell','Orthographic','Peters ','Polyconical','Putnins P1','Quartic Authalic','Robinson','Sanson-Flamsteed Sinusoidal',
	'Stereographic','Transverse Mercator','Tristan Edwards','Urmaev Flat-Polar Sinusoidal','Urmaev V','Van der Grinten I',
	'Wagner I (Kavraisky VI)','Wagner II','Wagner III','Wagner IV','Wagner VI','Wagner VII','Winkel I','Winkel II');

   function LoadErrorList(FileName:string):Boolean;
   function GetErrorCode(Error:integer):TErrorCode;

var
  ErrorListing : TErrorListing;

implementation
//------------------------------------------------------------------------------
function LoadErrorList(FileName:string):Boolean;
var
   Stream : TMemoryStream;
   I : Integer;
   R : SmallInt;
   P,E: Shortint;
   B : Byte;
begin
  if FileExists(FileName) then begin
    Stream := TMemoryStream.Create;
    Stream.LoadFromFile(FileName);

    for I := 0 to 999 do begin
      Stream.Read(B,SizeOf(Byte));
      if (B > 0) then begin
        Stream.Read(R,SizeOf(Smallint));
        Stream.Read(P,SizeOf(Shortint));
        Stream.Read(E,SizeOf(Shortint));

        ErrorListing[I,0] := P;
        ErrorListing[I,1] := E;
        ErrorListing[I,2] := R;
      end else begin
        ErrorListing[I,0] := -1;
        ErrorListing[I,1] := -1;
        ErrorListing[I,2] := -1;
      end;
    end;

    Stream.Free;
  end;
end;
//------------------------------------------------------------------------------
function GetErrorCode(Error:integer):TErrorCode;
begin
  Result.Routine    := '';
  Result.Projection := '';
  Result.Error      := '';
  if (Error >= 0) and (Error <= 999) then if (ErrorListing[Error,1] > -1) then begin
    Result.Projection := ErrorProject[ErrorListing[Error,0]];
    Result.Error      := ErrorText[ErrorListing[Error,1]];
    Result.Routine    := ErrorRoutine[ErrorListing[Error,2]];
  end;
end;
//------------------------------------------------------------------------------
end.
